# Databricks notebook source
# MAGIC %md
# MAGIC ## TOC
# MAGIC - 1. ~install~ quick fix: running as a notebook from within databricks. best practices is via *_runner.py.
# MAGIC - 2. Testing import
# MAGIC - 3. Testing python version compatibility
# MAGIC - 4. run community staging to check if the results match (use job synk_generate_all_community_staging)
# MAGIC - 5. upload to codeartifact or s3 for future use (done its at s3://*/internal/data_science/interim/tests/*.whl and can be used to install in DB clusters)

# COMMAND ----------

# MAGIC %md
# MAGIC ## 1: build and install

# COMMAND ----------

!python setup.py sdist bdist_wheel

# COMMAND ----------

# MAGIC %pip install --force-reinstall dist/sdutilities-1.2.5-py3-none-any.whl
# MAGIC dbutils.library.restartPython()

# COMMAND ----------

# MAGIC %md
# MAGIC ## 2. Testing imports

# COMMAND ----------

import sdutilities as sdu

# COMMAND ----------

# 
logger = sdu.SDLogger()#log_file=log_file, log_level=log_level)

# COMMAND ----------

# test_api was tested w/o best practices and hence running as a notebook and not as a pytest runner
# To Fix: create pytest around this notebook, and add asserts in api key file
#return_value = dbutils.notebook.run("tests/test_api_key", 600) # this fails for httpx issues. hence pasting all of the lines in this cell direclty
# assuming its passed if given output and assuming below is the right output. the original api test had no assert to indicate whats right oruput for this test

api_key = dbutils.secrets.get(scope="acs", key="api_key")
acs = sdu.ACSClient(2021, api_key=api_key)
acs_cols = ['B01001_001E']
acs.get_zips(acs_cols, sd_cols=True) 

# COMMAND ----------

# MAGIC %md
# MAGIC ## 3. create fake data, read, save, log, upsert  
# MAGIC - the tests/test_*.py only locally for now
# MAGIC - if local, then also run `>>pytest tests/test_package_compatibility.py` to check for python package and version compatibility on pypi

# COMMAND ----------

# MAGIC %md
# MAGIC ### 3.1 Test connection and create fake data

# COMMAND ----------

from sdutilities.datalink import DataLink

def test_get_connection_info():
    # Create an instance of DataLink with known configuration
    test_cfg = {
        'uname': 'testuser',
        'pw': 'testpassword',
        'host': 'testhost',
        'db': 'testdb'
    }
    datalink = DataLink()
    datalink._db_cfg = test_cfg  # Directly setting the config for testing

    # Call the method and get the result
    result = datalink.get_connection_info()

    # Assert that the result matches what we set
    assert result == test_cfg, "get_connection_info should return the correct configuration"

test_get_connection_info() # if no error assert does not print anything

# COMMAND ----------

from faker import Faker
import pandas as pd

fake = Faker()

# Example to create fake data for DataFrame
def create_fake_data(num_rows):
    data = {
        'column1': [fake.name() for _ in range(num_rows)],
        'column2': [fake.address() for _ in range(num_rows)],
        # Add more columns as needed
    }
    return pd.DataFrame(data)

fake_df = create_fake_data(10)  # Create a DataFrame with 10 rows of fake data

from unittest.mock import patch, MagicMock

fake_df.head(10)

# COMMAND ----------

# MAGIC %md
# MAGIC ## 4 Test community staging which only uses logger and awsclient
# MAGIC - 4.1 Test installing and importing the package via s3 (s3://*/internal/data_science/interim/tests/*.whl)
# MAGIC - 4.2 Test if the results match (use job synk_generate_all_community_staging)

# COMMAND ----------

# MAGIC %md
# MAGIC ## 5 Copy the package to s3 from local

# COMMAND ----------

dbutils.fs.rm("s3://sdac-s3-use1-data-ocean/silver/internal/users/data_science/interim/tests/sdutilities-1.2.5-py3-none-any.whl")

if False: # the below needs to done manually for it does not work in databricks. 
    dbutils.fs.cp("https://sociallydetermined-sdac-use1-databricks-core.cloud.databricks.com/?o=8603315037030455#files/1050444562685243", "dbfs:/tmp/")
    dbutils.fs.cp("dbfs:/tmp/sdutilities-1.2.5.tar.gz", "s3://sdac-s3-use1-data-ocean/silver/internal/users/data_science/interim/tests/")
else:
    print("1. export the wheel on local machine")
    print("2. upload the wheel on s3 here: 's3://sdac-s3-use1-data-ocean/silver/internal/users/data_science/interim/tests/'")


