# Databricks notebook source
# MAGIC %md
# MAGIC ## TOC
# MAGIC - 1. ~install~ quick fix: running as a notebook from within databricks. best practices is via *_runner.py.
# MAGIC - 2. import
# MAGIC - 3. create fake data, read, save, log, upsert
# MAGIC - 4. run community staging to check if the results match
# MAGIC - 5. upload to codeartifact for future use

# COMMAND ----------

!python setup.py sdist bdist_wheel

# COMMAND ----------

# MAGIC %pip install --force-reinstall dist/sdutilities-1.2.5-py3-none-any.whl
# MAGIC dbutils.library.restartPython()

# COMMAND ----------

# MAGIC %md
# MAGIC ## Testing directly the imports and its functionality

# COMMAND ----------

import sdutilities as sdu

# COMMAND ----------

# 
logger = sdu.SDLogger()#log_file=log_file, log_level=log_level)

# COMMAND ----------

# test_api was tested w/o best practices and hence running as a notebook and not as a pytest runner
# To Fix: create pytest around this notebook, and add asserts in api key file
#return_value = dbutils.notebook.run("tests/test_api_key", 600) # this fails for httpx issues. hence pasting all of the lines in this cell direclty
# assuming its passed if given output and assuming below is the right output. the original api test had no assert to indicate whats right oruput for this test

api_key = dbutils.secrets.get(scope="acs", key="api_key")
acs = sdu.ACSClient(2021, api_key=api_key)
acs_cols = ['B01001_001E']
acs.get_zips(acs_cols, sd_cols=True) 

# COMMAND ----------

import json

# Specify the path to the JSON file
json_file_path = "config.json"

# Add a test_cfg to the JSON data
update_json = {
    "test_cfg": {
        "drivername": "sqlite",
        "database": ":memory:",
        "echo": "false",
        "pw": "postgres2",
        "uname": "postgres"
    }
}

# Write the updated JSON data back to the file
with open(json_file_path, "w") as json_file:
    json.dump(update_json, json_file, indent=4)

# Read and display the updated JSON data
with open(json_file_path, "r") as json_file:
    updated_data = json.load(json_file)

print(updated_data)


# COMMAND ----------

from sdutilities import DataLink
from unittest.mock import MagicMock, patch
import pandas as pd
from faker import Faker

# Create a sample DataFrame for testing
fake = Faker()

def sample_df():
    """Creates a sample DataFrame with fake data for testing."""
    fake_data = {'col1': [fake.random_number(digits=3) for _ in range(10)],
                 'col2': [fake.name() for _ in range(10)],
                 'col3': [fake.date() for _ in range(10)]}
    return pd.DataFrame(fake_data)

def test_upsert():
    """Tests the upsert functionality with a sample DataFrame."""
    # Setup - create a table in the in-memory SQLite database
    mock_datalink = MagicMock()
    mock_datalink._engine = MagicMock()
    
    # Create a sample DataFrame
    sample_df_data = sample_df()

    try:
        # Create the table in the database
        sample_df_data.to_sql('test_table', con=mock_datalink._engine, index=False, if_exists='replace')

        # Test the upsert method
        DataLink.upsert(mock_datalink, sample_df_data, 'test_table', 'public')  # 'public' schema is a dummy value for SQLite

        # Verify that the data is upserted correctly
        result_df = pd.read_sql_table('test_table', con=mock_datalink._engine)
        assert result_df.equals(sample_df_data)
    except Exception as e:
        print(f"Error: {str(e)}")
    return print(f"upsert works")

# Run the tests
test_upsert()

# COMMAND ----------

import pandas as pd
from faker import Faker
from sdutilities import DataLink

# Initialize Faker
fake = Faker()

def generate_and_upsert_fake_data(num_records=10):
    # Create a DataFrame with fake data
    fake_data = {
        "name": [fake.name() for _ in range(num_records)],
        "email": [fake.email() for _ in range(num_records)],
        "phone_number": [fake.phone_number() for _ in range(num_records)]
    }
    df = pd.DataFrame(fake_data)

    # Create a DataLink instance (modify the arguments as needed)
    datalink = DataLink()#cfg_name="your_config_name", db_file="path/to/your/db_cfg.json")

    # Upsert the DataFrame into a table named "fake_data"
    datalink.upsert(df, table="fake_data", schema="public", constraint_cols=["name"])

    # Display the results from the table
    sql = "SELECT * FROM public.fake_data"
    result_df = datalink.sql_to_df(sql)
    print("Results from the 'fake_data' table:")
    display(result_df)

# Call the function to generate and upsert fake data
generate_and_upsert_fake_data(num_records=5)



# COMMAND ----------

# MAGIC %%writefile test_datalink.py
# MAGIC
# MAGIC # Paste your test function here
# MAGIC import pytest
# MAGIC from sdutilities.datalink import DataLink  # Replace 'your_module' with the actual module name
# MAGIC from unittest.mock import MagicMock, patch
# MAGIC import pandas as pd
# MAGIC from faker import Faker
# MAGIC
# MAGIC # Create a sample DataFrame for testing
# MAGIC fake = Faker()
# MAGIC @pytest.fixture
# MAGIC def sample_df():
# MAGIC     """Creates a sample DataFrame with fake data for testing."""
# MAGIC     fake_data = {'col1': [fake.random_number(digits=3) for _ in range(10)],
# MAGIC                  'col2': [fake.name() for _ in range(10)],
# MAGIC                  'col3': [fake.date() for _ in range(10)]}
# MAGIC     return pd.DataFrame(fake_data)
# MAGIC
# MAGIC def test_upsert(mock_datalink, sample_df):
# MAGIC     """Tests the upsert functionality with a sample DataFrame."""
# MAGIC     # Setup - create a table in the in-memory SQLite database
# MAGIC     sample_df.to_sql('test_table', con=mock_datalink._engine, index=False, if_exists='replace')
# MAGIC
# MAGIC     # Test the upsert method
# MAGIC     mock_datalink.upsert(sample_df, 'test_table', 'public')  # 'public' schema is a dummy value for SQLite
# MAGIC
# MAGIC     # Verify that the data is upserted correctly
# MAGIC     result_df = pd.read_sql_table('test_table', con=mock_datalink._engine)
# MAGIC     assert result_df.equals(sample_df)
# MAGIC

# COMMAND ----------


def test_get_connection_info():
    # Create an instance of DataLink with known configuration
    test_cfg = {
        'uname': 'testuser',
        'pw': 'testpassword',
        'host': 'testhost',
        'db': 'testdb'
    }
    datalink = DataLink()
    datalink._db_cfg = test_cfg  # Directly setting the config for testing

    # Call the method and get the result
    result = datalink.get_connection_info()

    # Assert that the result matches what we set
    assert result == test_cfg, "get_connection_info should return the correct configuration"

test_get_connection_info() # if no error assert does not print anything

# COMMAND ----------

from faker import Faker
import pandas as pd

fake = Faker()

# Example to create fake data for DataFrame
def create_fake_data(num_rows):
    data = {
        'column1': [fake.name() for _ in range(num_rows)],
        'column2': [fake.address() for _ in range(num_rows)],
        # Add more columns as needed
    }
    return pd.DataFrame(data)

fake_df = create_fake_data(10)  # Create a DataFrame with 10 rows of fake data

from unittest.mock import patch, MagicMock


# COMMAND ----------

def test_upsert(fake_df):
    datalink = DataLink()  # Assuming DataLink is properly initialized
    
    with patch('sqlalchemy.create_engine') as mock_create_engine:
        mock_engine = MagicMock()
        mock_create_engine.return_value = mock_engine

        # Mock additional methods if required
        # ...

        # Call the upsert function
        datalink.upsert(fake_df, 'fake_table', 'fake_schema', ['column1'])

        # You can add assertions here to check if the correct SQL commands were executed
        # For example:
        # mock_engine.execute.assert_called_with(...)
test_upsert()


# COMMAND ----------

# MAGIC %md
# MAGIC ## Databricks cache issues to run all of the below code

# COMMAND ----------

import os
import sys

_ = %pwd
print(f"current location {_}")

__file__ = "/Workspace/Repos/cdalal@sdoh.private/sdutilities_package"
# Get the path to the directory for this file in the workspace.
dir_root = __file__ #os.path.dirname(os.path.realpath(__file__))
# Switch to the root directory.
os.chdir(dir_root)
_ = %pwd
print(f"change location to {_}")

# Skip writing .pyc files to the bytecode cache on the cluster.
sys.dont_write_bytecode = True


# COMMAND ----------

# %pip install --upgrade typing-extensions pytest
# dbutils.library.restartPython()

# COMMAND ----------

import pytest
retcode = pytest.main(["/Workspace/Users/cdalal@sdoh.private/test_package_compatibility.py", "-p", "no:cacheprovider"])

# Fail the cell execution if we have any test failures.
assert retcode == 0, 'The pytest invocation failed. See the log above for details.'

# COMMAND ----------

import subprocess

p = subprocess.Popen(['pytest', '/Workspace/Users/cdalal@sdoh.private/test_package_compatibility.py', '-p', 'no:cacheprovider'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)

out, err = p.communicate()
retcode = p.returncode

print('----------------')
print(out.decode())
print('----------------')
print(err.decode())
print('----------------')
print(f"Return code: {retcode}")

# Fail the cell execution if we have any test failures.
if retcode != 0:
    print(f"The pytest invocation failed with return code {retcode}. Below is the error message and log output:")
    print('----------------')
    print(err.decode())
    print(out.decode())
    print('----------------')
    raise AssertionError('One or more tests have failed. See the log above for details.')

# COMMAND ----------

!pytest --cache-clear

# COMMAND ----------

!pytest -sq

# COMMAND ----------

!pytest -p no:cacheprovider tests/test_package_compatibility.py

# COMMAND ----------

!pytest tests/test_datalink.py

# COMMAND ----------


